<?php
/**
 * @file
 * @package sd
 * @version $Id$
**/

if(!defined('XOOPS_ROOT_PATH'))
{
    exit;
}

/**
 * Sd_ItemObject
**/
class Sd_ItemObject extends XoopsSimpleObject
{
    /**
     * Sd_ItemObject
     * 
     * @param   void
     * 
     * @return  void
     * 
     * @public
    **/
    function Sd_ItemObject()
    {
        $this->initVar('item_id', XOBJ_DTYPE_INT, '', false);
        $this->initVar('item_name', XOBJ_DTYPE_STRING, '', false, 255);
        $this->initVar('kind', XOBJ_DTYPE_INT, '', false);
        $this->initVar('lv', XOBJ_DTYPE_INT, '', false);
        $this->initVar('element', XOBJ_DTYPE_INT, '', false);
        $this->initVar('at', XOBJ_DTYPE_INT, '', false);
        $this->initVar('df', XOBJ_DTYPE_INT, '', false);
        $this->initVar('mat', XOBJ_DTYPE_INT, '', false);
        $this->initVar('mdf', XOBJ_DTYPE_INT, '', false);
        $this->initVar('range', XOBJ_DTYPE_INT, '', false);
        $this->initVar('regist_time', XOBJ_DTYPE_INT, '', false);
    }
}

/**
 * Sd_ItemHandler
**/
class Sd_ItemHandler extends XoopsObjectGenericHandler
{
    /**
     * @var  string
     * 
     * @public
    **/
    var $mTable = '{dirname}_item';

    /**
     * @var  string
     * 
     * @public
    **/
    var $mPrimary = 'item_id';

    /**
     * @var  string
     * 
     * @public
    **/
    var $mClass = 'Sd_ItemObject';

    /**
     * @brief   string
    **/
    protected $_mSlotTable = '{dirname}_item_slot';

    /**
     * @brief   string
    **/
    protected $_mDropTable = '{dirname}_drop';

    /**
     * Sd_ItemHandler
     * 
     * @param   XoopsDatabase  &$db
     * @param   string  $dirname
     * 
     * @return  void
     * 
     * @public
    **/
    function Sd_ItemHandler(&$db,$dirname)
    {
        $this->mTable = str_replace('{dirname}',$dirname,$this->mTable);
        parent::XoopsObjectGenericHandler($db);
        
        $this->_mSlotTable = $this->db->prefix(str_replace('{dirname}',$dirname,$this->_mSlotTable));
        $this->_mDropTable = $this->db->prefix(str_replace('{dirname}',$dirname,$this->_mDropTable));
    }
    
    /**
     * escapeString
     * 
     * @param   string  $str
     * 
     * @return  string
    **/
    protected function escapeString(/*** string ***/ $str)
    {
        return mysql_real_escape_string($str,$this->db->conn);
    }
    
    /**
     * getByName
     * 
     * @param   string  $name
     * 
     * @return  Sd_ItemObject
    **/
    public function getByName(/*** string ***/ $name)
    {
        $ret = $this->getObjects(new Criteria('item_name',$name));
        
        if(is_array($ret) && count($ret) == 1)
        {
            return $ret[0];
        }
        
        return null;
    }
    
    /**
     * &getItemList
     * 
     * @param   CriteriaElement $cri
     * 
     * @return  Sd_ItemObject[]
    **/
    public function &getItemList(CriteriaElement $cri)
    {
        $ret = array();
        $sql = XCube_Utils::formatString(
            'select `i`.*,'
          . '`d1`.`drop_name` as s1_name,`d1`.`gift` as s1_gift,'
          . '`d2`.`drop_name` as s2_name,`d2`.`gift` as s2_gift,'
          . '`d3`.`drop_name` as s3_name,`d3`.`gift` as s3_gift '
          . 'from `{0}` as i '
          . 'left join `{1}` as s1 on `i`.`item_id` = `s1`.`item_id` and `s1`.`slot` = 1 '
          . 'left join `{2}` as d1 on `d1`.`drop_id` = `s1`.`drop_id` '
          . 'left join `{1}` as s2 on `i`.`item_id` = `s2`.`item_id` and `s2`.`slot` = 2 '
          . 'left join `{2}` as d2 on `d2`.`drop_id` = `s2`.`drop_id` '
          . 'left join `{1}` as s3 on `i`.`item_id` = `s3`.`item_id` and `s3`.`slot` = 3 '
          . 'left join `{2}` as d3 on `d3`.`drop_id` = `s3`.`drop_id` ',
            $this->mTable,$this->_mSlotTable,$this->_mDropTable
        );
        $where = '';
        
        for($i = 0;$i < $cri->getCountChildElements();$i++)
        {
            $c = $cri->getChildElement($i);
            switch($c->getName())
            {
                case 'item_name':
                    $where .= XCube_Utils::formatString(
                        "{0}`i`.`item_name` like '{2}%' ",
                        ($where != null ? 'and ' : ''),$this->escapeString($c->getValue())
                    );
                    break;
                case 'kind':
                case 'lv':
                case 'element':
                case 'at':
                case 'df':
                case 'mat':
                case 'mdf':
                case 'range':
                case 'regist_time':
                    $where .= XCube_Utils::formatString(
                        "{0}`i`.`{1}` {2} {3} ",
                        ($where != null ? 'and ' : ''),$c->getName(),$c->getOperator(),intval($c->getValue())
                    );
                    break;
                case 'gift':
                    $sql .= XCube_Utils::formatString(
                        'left join `{0}` as w{1} on `i`.`item_id` = `w{1}`.`item_id` '
                      . 'left join `{2}` as n{1} on `n{1}`.`drop_id` = `w{1}`.`drop_id` ',
                        $this->_mSlotTable,$i,$this->_mDropTable
                    );
                    $where .= XCube_Utils::formatString(
                        "{0}`n{1}`.`gift` like '{2}%' ",
                        ($where != null ? 'and ' : ''),$i,$this->escapeString($c->getValue())
                    );
                    break;
                default:
                    break;
            }
        }
        
        $sql .= ($where != null ? 'where ' . $where : '') . 'group by `i`.`item_id` order by `i`.`item_name` asc';
        
        if($res = $this->db->query($sql,$cri->getLimit(),$cri->getStart()))
        {
            while($row = $this->db->fetchArray($res))
            {
                $obj = $this->create();
                $obj->initVar('s1_name',XOBJ_DTYPE_STRING,'',false,255);
                $obj->initVar('s1_gift',XOBJ_DTYPE_STRING,'',false,255);
                $obj->initVar('s2_name',XOBJ_DTYPE_STRING,'',false,255);
                $obj->initVar('s2_gift',XOBJ_DTYPE_STRING,'',false,255);
                $obj->initVar('s3_name',XOBJ_DTYPE_STRING,'',false,255);
                $obj->initVar('s3_gift',XOBJ_DTYPE_STRING,'',false,255);
                $obj->setVars($row);
                $ret[] =& $obj;
                unset($obj);
            }
        }
        
        return $ret;
    }
    
    /**
     * getItemListCount
     * 
     * @param   CriteriaElement $cri
     * 
     * @return  int
    **/
    public function getItemListCount(CriteriaElement $cri)
    {
        $sql = 'select count(distinct `i`.`item_id`) as cnt from `' . $this->mTable . '` as i ';
        $where = '';
        
        for($i = 0;$i < $cri->getCountChildElements();$i++)
        {
            $c = $cri->getChildElement($i);
            switch($c->getName())
            {
                case 'item_name':
                    $where .= XCube_Utils::formatString(
                        "{0}`i`.`item_name` like '{2}%' ",
                        ($where != null ? 'and ' : ''),$this->escapeString($c->getValue())
                    );
                    break;
                case 'kind':
                case 'lv':
                case 'element':
                case 'at':
                case 'df':
                case 'mat':
                case 'mdf':
                case 'range':
                case 'regist_time':
                    $where .= XCube_Utils::formatString(
                        "{0}`i`.`{1}` {2} {3} ",
                        ($where != null ? 'and ' : ''),$c->getName(),$c->getOperator(),intval($c->getValue())
                    );
                    break;
                case 'gift':
                    $sql .= XCube_Utils::formatString(
                        'left join `{0}` as w{1} on `i`.`item_id` = `w{1}`.`item_id` '
                      . 'left join `{2}` as n{1} on `n{1}`.`drop_id` = `w{1}`.`drop_id` ',
                        $this->_mSlotTable,$i,$this->_mDropTable
                    );
                    $where .= XCube_Utils::formatString(
                        "{0}`n{1}`.`gift` like '{2}%' ",
                        ($where != null ? 'and ' : ''),$i,$this->escapeString($c->getValue())
                    );
                    break;
                default:
                    break;
            }
        }
        
        if($where != null)
        {
            $sql .= 'where ' . $where;
        }
        
        if($res = $this->db->query($sql))
        {
            $row = $this->db->fetchArray($res);
            return $row['cnt'];
        }
        
        return null;
    }
}

?>
